﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Practices.Prism.Regions;
using System.Windows.Controls.Ribbon;

namespace Host.Infrastructure.RegionAdapters
{
    public class RibbonRegionAdapter : RegionAdapterBase<Ribbon>
    {
        public RibbonRegionAdapter(IRegionBehaviorFactory behaviorFactory)
            : base(behaviorFactory)
        {
        }

        protected override void Adapt(IRegion region, Ribbon regionTarget)
        {
            region.Views.CollectionChanged += (sender, e) =>
                {
                    switch (e.Action)
                    {
                        case NotifyCollectionChangedAction.Add:
                            foreach (FrameworkElement element in e.NewItems)
                            {
                                if (element is RibbonTab)
                                {
                                    RibbonTab newRibbonTab = element as RibbonTab;
                                    //regionTarget.Items.Add(element);
                                    if (!string.IsNullOrEmpty(Convert.ToString(newRibbonTab.Header)))
                                    {
                                        RibbonTab existingRibbonTab = (from RibbonTab tab in regionTarget.Items where 
                                                                       String.CompareOrdinal(Convert.ToString(tab.Header),Convert.ToString(newRibbonTab.Header)) == 0
                                                                       select tab).FirstOrDefault();

                                        // tab doesn't exist, add tab
                                        if (existingRibbonTab == null)
                                            regionTarget.Items.Add(newRibbonTab);
                                        else
                                        {
                                            List<object> ribbonGroups = new List<object>();
                                            foreach (RibbonGroup newGroup in newRibbonTab.Items)
                                            {
                                                RibbonGroup existingTabGroup = (from RibbonGroup grp in existingRibbonTab.Items where
                                                            String.CompareOrdinal(Convert.ToString(grp.Header), Convert.ToString(newGroup.Header)) == 0
                                                            select grp).FirstOrDefault();
                                                // group does not exist, add it to a temp list to remove from new tab and add in existing tab
                                                if (existingTabGroup == null)
                                                    ribbonGroups.Add(newGroup); 
                                                else
                                                {
                                                    //if group exists, then remove items from this newTab->group and it it to existing tab->group
                                                    List<object> ribbonItems = newGroup.Items.Cast<object>().ToList();

                                                    foreach (var newItem in ribbonItems)
                                                    {
                                                        newGroup.Items.Remove(newItem);
                                                        existingTabGroup.Items.Add(newItem);
                                                    }
                                                }
                                            }
                                            // remove from new tab and add in existing tab
                                            foreach(var grp in ribbonGroups)
                                            {
                                                newRibbonTab.Items.Remove(grp);
                                                existingRibbonTab.Items.Add(grp);
                                            }
                                        }
                                    }
                                }
                            }
                            break;

                        case NotifyCollectionChangedAction.Remove:
                            foreach (UIElement elementLoopVariable in e.OldItems)
                            {
                                var element = elementLoopVariable;
                                if (regionTarget.Items.Contains(element))
                                {
                                    regionTarget.Items.Remove(element);
                                }
                            }
                            break;
                    }
                };
            //// If control has child items, move them to the region and then bind control to region. Can't set ItemsSource if child items exist.
            //if (regionTarget.Items.Count > 0)
            //{
            //    foreach (object childItem in regionTarget.Items)
            //    {
            //        region.Add(childItem);
            //    }
            //    // Control must be empty before setting ItemsSource
            //    regionTarget.Items.Clear();
            //}
            //// thanks to this operation, ViewHintSort is taken into accout!
            //regionTarget.ItemsSource = region.Views;
        }

        protected override IRegion CreateRegion()
        {
            return new AllActiveRegion();
        }
    }
}
