﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls.Ribbon;
using Host.Infrastructure.Behaviour;
using Host.Infrastructure.RegionAdapters;
using Infragistics.Windows.DockManager;
using Microsoft.Practices.Prism.MefExtensions;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Prism.Regions;


namespace ACVS.Enterprise.Client.Core.Host
{
    class HostBootstrapper : MefBootstrapper
    {
        //private readonly UserProfileService userProfileService = new UserProfileService();

        protected override void ConfigureAggregateCatalog()
        {

            this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(HostBootstrapper).Assembly));
            this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(AutoPopulateExportedViewsBehavior).Assembly));
            //this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(UserProfileService).Assembly));
            // These modules are not referenced in the project and are discovered by inspecting a directory.
            //DirectoryCatalog catalog = new DirectoryCatalog(".\\Modules");
            //this.AggregateCatalog.Catalogs.Add(catalog);
            string modulesDirectory = ".\\Modules";
            if (Directory.Exists(modulesDirectory))
            {
                // Recurse into subdirectories of this directory.
                string[] subdirectoryEntries = Directory.GetDirectories(modulesDirectory);
                // Add all module directories in catalog
                foreach (string subdirectory in subdirectoryEntries)
                    this.AggregateCatalog.Catalogs.Add(new DirectoryCatalog(subdirectory));
            }
        }

        protected override DependencyObject CreateShell()
        {
            return this.Container.GetExportedValue<ShellWindow>();
        }

        protected override void InitializeShell()
        {
            base.InitializeShell();

            Application.Current.MainWindow = (ShellWindow)this.Shell;
            Application.Current.MainWindow.Show();

        }

        protected override IRegionBehaviorFactory ConfigureDefaultRegionBehaviors()
        {
            var factory = base.ConfigureDefaultRegionBehaviors();
            factory.AddIfMissing("AutoPopulateExportedViewsBehavior", typeof(AutoPopulateExportedViewsBehavior));
            return factory;
        }

        protected override RegionAdapterMappings ConfigureRegionAdapterMappings()
        {
            IRegionBehaviorFactory factory = this.Container.GetExportedValue<IRegionBehaviorFactory>();
            
            RegionAdapterMappings mappings = base.ConfigureRegionAdapterMappings();
            mappings.RegisterMapping(typeof(TabGroupPane), new TabGroupPaneRegionAdapter(factory));
            mappings.RegisterMapping(typeof(Ribbon), new RibbonRegionAdapter(factory));
            return mappings;
        }
        
        /// <summary>
        /// Creates the <see cref="IModuleCatalog"/> used by Prism.
        /// </summary>
        /// <remarks>
        /// The base implementation returns a new ModuleCatalog.
        /// </remarks>
        /// <returns>
        /// A ConfigurationModuleCatalog.
        /// </returns>
        protected override IModuleCatalog CreateModuleCatalog()
        {
            // When using MEF, the existing Prism ModuleCatalog is still the place to configure modules via configuration files.
            return new ConfigurationModuleCatalog();
        }

        protected override void ConfigureContainer()
        {
            base.ConfigureContainer();
            // Because we created the CallbackLogger and it needs to be used immediately, we compose it to satisfy any imports it has.
            //this.Container.ComposeExportedValue<UserProfileService>(this.userProfileService);
        }
    }
}
